Struct representing a modbus mapping.
Summary
Functions
Get the Range of addresses for the given ModBoss.Mapping
Returns a map describing the gap between two mappings of the same type.
Guard that checks whether two mappings are of the same type and occupy adjacent addresses.
Types
@type address() :: non_neg_integer()
@type count() :: pos_integer()
@type name() :: atom()
@type object_type() :: :holding_register | :input_register | :coil | :discrete_input
Functions
Get the Range of addresses for the given ModBoss.Mapping
Returns a map describing the gap between two mappings of the same type.
Returns %{size: integer(), addresses: MapSet.t()} where addresses contains
{type, address} pairs for each address in the gap.
Guard that checks whether two mappings are of the same type and occupy adjacent addresses.